<?php
class PostsController extends AppController {

  public  $name = 'Posts';
  
//==================================================================================================
function __CALL_BACK_FUNCTIONS() {}
//==================================================================================================

//==================================================================================================
  // Executed before every action involving the users.
  public function beforeFilter() {
    parent::beforeFilter();
  }
  
//==================================================================================================
function __ACTIVE_ACTIONS() {}
//==================================================================================================

//==================================================================================================
  // Lists a group of posts. The set of posts can be all if there are no parameters, found by city
  //   and state if parameters are of the form /City/State, for a particular state if the state_id
  //   parameter is passed, or for a particular category if the cat_id parameter is passed.
  public function listing() {
    // TODO: Find the Cake way to perform this find (actually the two branches of it as appended
    //   below).
    $sql = 'SELECT  Post.*, ZipCode.*, State.*, User.*, PostCategory.*
            FROM    posts AS Post,
                    zip_codes AS ZipCode,
                    states AS State,
                    users AS User,
                    post_categories AS PostCategory
            WHERE   Post.zip_code_id = ZipCode.id
            AND     ZipCode.state_id = State.id
            AND     Post.user_id = User.id
            AND     Post.post_category_id = PostCategory.id';
    
    // If the the params includes passed params, perform a custom query based on the city and state.
    if ( !empty( $this->params['pass'] ) ) {
      $city = $this->params['pass'][0];
      $state = $this->params['pass'][1];
      
      $sql .= "\nAND     ZipCode.city = '$city'
                 AND     State.state = '$state'
                 LIMIT   10;";
      
      $this->set( 'posts', $this->Post->query( $sql ) );
      
    // If the params includes a named param, find based on the named parameter.
    } elseif ( !empty( $this->params['named'] ) ) {
      
      if ( isset( $this->params['named']['cat_id'] ) ) {
        $this->set( 'posts', $this->Post->find( 'all', array(
          'conditions' => array( 'Post.post_category_id' => $this->params['named']['cat_id'] ) ) ) );
          
      } elseif( isset( $this->params['named']['state_id'] ) ) {
        $stateID = $this->params['named']['state_id'];
        $sql .= "\nAND    Post.post_category_id = PostCategory.id
                   AND    State.id = $stateID
                   LIMIT  10";

        $this->set( 'posts', $this->Post->query( $sql ) );
      }
      
    // Otherwise find all of the post records.
    } else  {
      $this->set( 'posts', $this->Post->find( 'all', array( 'limit' => '10' ) ) );
    }
  }

//==================================================================================================
  // Creates a new post record from the form data passed in $this->data, pushing media files to the
  //   appropriate database and storing them in the correct location.
  public function create() {
    if ( !empty( $this->data ) ) {  
      $this->Post->create();
      
      if ( $this->Post->save( $this->data ) ) {
        // Check for file uploads.
        $this->loadModel( 'MediaFile' );
        $mediaIDs = array();
        foreach ( array( 'image', 'audio', 'video' ) as $mediaType ) {
          if ( is_uploaded_file( $this->data['Post'][$mediaType]['tmp_name'] ) ) {
            $this->MediaFile->create_record_and_move_file( $this->data['Post'][$mediaType],
                                                           $mediaType,
                                                           'post',
                                                           $this->Post->id );
          }
        }

        $this->Session->setFlash( ___( 'The demand post has been created.' ) );
        $this->redirect( array( 'action' => 'track', $this->Post->id ) );
        
      } else {
        $this->Session->setFlash( ___( 'The demand post could not be saved. Please, try again.' ) );
      }
    }
    
    $this->set( 'postCategories', $this->Post->PostCategory->find( 'list' ) );
  }

//==================================================================================================
  // Used to get the details of a single post.
  public function track( $id = null ) {
    if ( !$id ) {
      $this->redirect( array( 'action' => 'listing' ) );

    } else {
      $this->Post->id = $id;
      $post = $this->Post->read();
      
      $this->set( array( 'post'         => $post,
                         'zipCode'      => $this->Post->ZipCode->find( 'first' ),
                         'votesUpMsg'   => $this->Post->votes_up_str(),
                         'votesDownMsg' => $this->Post->votes_down_str(),
                         'timeLeftStr'  => $this->Post->pledge_time_left() ) );

    }
  }

//==================================================================================================
  // Outputs either the most recent demands or the most popular demands for the given period.
  //   Defaults to the most recent demands.
  public function top_demands( $period = null ) {
    if ( !$period ) {
      $period = 'most_recent';
    }
    $this->set( 'period', $period );
    
    if ( $period == 'most_recent' ) {
      $this->set( 'posts', $this->Post->most_recent_posts() );
    } else {
      $this->set( 'posts', $this->Post->top_demands( $period ) );
    }
  }

//==================================================================================================
function __ADMIN_ACTIONS() {}
//==================================================================================================

//==================================================================================================
  public function admin_index() {
    $this->Post->recursive = 0;
    $this->set( 'posts', $this->paginate() );
  }

//==================================================================================================
  public function admin_view( $id = null ) {
    if ( !$id ) {
      $this->Session->setFlash( ___( 'Invalid post' ) );
      $this->redirect( array( 'action' => 'index' ) );

    } else {
      $this->set( 'post', $this->Post->read( null, $id ) );
    }
  }

//==================================================================================================
  public function admin_add() {
    if ( !empty( $this->data ) ) {
      $this->Post->create();

      if ( $this->Post->save( $this->data ) ) {
        $this->Session->setFlash( ___( 'The post has been saved' ) );
        $this->redirect( array( 'action' => 'index' ) );

      } else {
        $this->Session->setFlash( ___( 'The post could not be saved. Please, try again.' ) );
      }
    }

    $users = $this->Post->User->find( 'list' );
    $postCategories = $this->Post->PostCategory->find( 'list' );
    $this->set( compact( 'users', 'postCategories' ) );
  }

//==================================================================================================
  public function admin_edit( $id = null ) {

    if ( !$id && empty( $this->data ) ) {
      $this->Session->setFlash( ___( 'Invalid post' ) );
      $this->redirect( array( 'action' => 'index' ) );
    }

    if ( !empty( $this->data ) ) {

      if ( $this->Post->save( $this->data ) ) {
        $this->Session->setFlash( ___( 'The post has been saved' ) );
        $this->redirect( array( 'action' => 'index' ) );

      } else {
        $this->Session->setFlash( ___( 'The post could not be saved. Please, try again.' ) );
      }
    }

    if ( empty( $this->data ) ) {
      $this->data = $this->Post->read( null, $id );
    }

    $users = $this->Post->User->find( 'list' );
    $postCategories = $this->Post->PostCategory->find( 'list' );
    $this->set( compact( 'users', 'postCategories' ) );
  }

//==================================================================================================
  public function admin_delete( $id = null ) {
    if ( !$id ) {
      $this->Session->setFlash( ___( 'Invalid id for post' ) );
      $this->redirect( array( 'action'=>'index' ) );
    }

    if ( $this->Post->delete( $id ) ) {
      $this->Session->setFlash( ___( 'Post deleted' ) );
      $this->redirect( array( 'action'=>'index' ) );
    }

    $this->Session->setFlash( ___( 'Post was not deleted' ) );
    $this->redirect( array( 'action' => 'index' ) );
  }

//==================================================================================================
function __PRIVATE_METHODS() {}
//==================================================================================================

}
?>